@@ -1987,11 +1987,14 @@ int coda_sp3_close(coda_product *product)
     {
         coda_dynamic_type_delete(product->root_type);
     }
-
     if (product->filename != NULL)
     {
         free(product->filename);
     }
+    if (product->mem_ptr != NULL)
+    {
+        free(product->mem_ptr);
+    }
 
     free(product);
 
