@@ -97,6 +97,7 @@ struct za_file_struct
     int fd;
     int num_entries;
     char *filename;
+    int64_t file_size;
     za_entry *entry;
     hashtable *hash_data;
     void (*handle_error) (const char *, ...);
@@ -113,6 +114,7 @@ static int get_entries(za_file *zf)
     uint32_t signature;
     uint32_t offset;
     uint16_t num_entries;
+    long result;
     int i;
 
     /* read the 'end of central directory record' */
@@ -179,11 +181,17 @@ static int get_entries(za_file *zf)
         entry = &zf->entry[i];
 
         /* read the constant length part of the central directory 'file header' */
-        if (read(zf->fd, buffer.as_int8, 46) < 0)
+        result = read(zf->fd, buffer.as_int8, 46);
+        if (result < 0)
         {
             zf->handle_error(strerror(errno));
             return -1;
         }
+        if (result != 46)
+        {
+            entry->zf->handle_error("unexpected end in zip file");
+            return -1;
+        }
 
         signature = buffer.as_uint32[0];
 #ifdef WORDS_BIGENDIAN
@@ -277,11 +285,17 @@ static int get_entries(za_file *zf)
             return -1;
         }
 
-        if (read(zf->fd, entry->filename, entry->filename_length) < 0)
+        result = read(zf->fd, entry->filename, entry->filename_length);
+        if (result < 0)
         {
             zf->handle_error(strerror(errno));
             return -1;
         }
+        if (((uint32_t)result) != entry->filename_length)
+        {
+            entry->zf->handle_error("unexpected end in zip file");
+            return -1;
+        }
         entry->filename[entry->filename_length] = '\0';
 
         if (hashtable_add_name(zf->hash_data, entry->filename) != 0)
@@ -296,6 +310,41 @@ static int get_entries(za_file *zf)
             zf->handle_error(strerror(errno));
             return -1;
         }
+
+        /* perform some quick sanity checks on the offsets and sizes */
+        if (entry->localheader_offset > zf->file_size)
+        {
+            entry->zf->handle_error("invalid zip file (local header offset exceeds file size)");
+            return -1;
+        }
+        if (entry->compression != 0)
+        {
+            if (entry->localheader_offset + entry->compressed_size > zf->file_size)
+            {
+                entry->zf->handle_error("invalid zip file (entry size exceeds file size)");
+                return -1;
+            }
+            /* for zlib the theoretical maximum compression factor is 1032 */
+            if (entry->uncompressed_size / 1032 > entry->compressed_size + 1)
+            {
+                entry->zf->handle_error("invalid uncompressed size in zip file");
+                return -1;
+            }
+        }
+        else
+        {
+            if (entry->compressed_size != entry->uncompressed_size)
+            {
+                entry->zf->handle_error("invalid zip file (compressed and uncompressed sizes should be equal for "
+                                        "uncompressed entry)");
+                return -1;
+            }
+            if (entry->localheader_offset + entry->uncompressed_size > zf->file_size)
+            {
+                entry->zf->handle_error("invalid zip file (entry size exceeds file size)");
+                return -1;
+            }
+        }
     }
 
     return 0;
@@ -350,6 +399,7 @@ za_file *za_open(const char *filename, void (*error_handler) (const char *, ...)
         }
         return NULL;
     }
+    zf->file_size = statbuf.st_size;
     zf->num_entries = 0;
     zf->entry = NULL;
     zf->hash_data = NULL;
@@ -381,7 +431,7 @@ za_file *za_open(const char *filename, void (*error_handler) (const char *, ...)
         za_close(zf);
         return NULL;
     }
-    if (buffer[0] != 'P' && buffer[1] != 'K')
+    if (buffer[0] != 'P' || buffer[1] != 'K')
     {
         error_handler("could not open %s (not a zip file)", filename);
         za_close(zf);
@@ -457,6 +507,7 @@ int za_read_entry(za_entry *entry, char *out_buffer)
     uint32_t uncompressed_size;
     uint16_t filename_length;
     uint16_t extrafield_length;
+    long result;
 
     if (lseek(entry->zf->fd, entry->localheader_offset, SEEK_SET) < 0)
     {
@@ -465,11 +516,17 @@ int za_read_entry(za_entry *entry, char *out_buffer)
     }
 
     /* read the constant length part of the 'local file header' */
-    if (read(entry->zf->fd, buffer.as_int8, 30) < 0)
+    result = read(entry->zf->fd, buffer.as_int8, 30);
+    if (result < 0)
     {
         entry->zf->handle_error(strerror(errno));
         return -1;
     }
+    if (result != 30)
+    {
+        entry->zf->handle_error("unexpected end in zip file");
+        return -1;
+    }
 
     signature = buffer.as_uint32[0];
 #ifdef WORDS_BIGENDIAN
@@ -577,11 +634,17 @@ int za_read_entry(za_entry *entry, char *out_buffer)
 
     if (entry->compression == 0)
     {
-        if (read(entry->zf->fd, out_buffer, entry->uncompressed_size) < 0)
+        result = read(entry->zf->fd, out_buffer, entry->uncompressed_size);
+        if (result < 0)
         {
             entry->zf->handle_error(strerror(errno));
             return -1;
         }
+        if (((uint32_t)result) != entry->uncompressed_size)
+        {
+            entry->zf->handle_error("unexpected end in zip file");
+            return -1;
+        }
     }
     else
     {
@@ -596,12 +659,19 @@ int za_read_entry(za_entry *entry, char *out_buffer)
             return -1;
         }
 
-        if (read(entry->zf->fd, in_buffer, entry->compressed_size) < 0)
+        result = read(entry->zf->fd, in_buffer, entry->compressed_size);
+        if (result < 0)
         {
             entry->zf->handle_error(strerror(errno));
             free(in_buffer);
             return -1;
         }
+        if (((uint32_t)result) != entry->compressed_size)
+        {
+            entry->zf->handle_error("unexpected end in zip file");
+            free(in_buffer);
+            return -1;
+        }
 
         zs.next_in = Z_NULL;
         zs.avail_in = 0;
@@ -659,6 +729,11 @@ int za_read_entry(za_entry *entry, char *out_buffer)
             }
             return -1;
         }
+        if (zs.avail_in != 0 || zs.avail_out != 0)
+        {
+            entry->zf->handle_error("invalid compressed/uncompressed size for zip entry");
+            return -1;
+        }
     }
 
     return 0;
